@@ -74,13 +74,13 @@ class ItemsGenerator(LoggerMixin):
     """Creates items for the components Client, Connection(s), Host(s) and Agent."""
 
     def __init__(self, main_window, configuration):
-        """Initialize the ShapeGenerator object."""
+        """Instantiates the ShapeGenerator object."""
         super().__init__()
         self.main_window = main_window
         self.config = configuration
         self.status = main_window.main_frame.canvas_frame.canvas_status
         self.canvas = main_window.main_frame.canvas_frame.canvas_landscape
-        self.scale = main_window.scale
+        # self.scale = main_window.scale  # IN DEVELOPMENT
         self.iteration = 0
         self.quit = False
         self._wipe_canvas()
@@ -104,15 +104,15 @@ class ItemsGenerator(LoggerMixin):
             • element 3: agent - an Agent is a 'child' of the target destination host
 
         """
-        start_y_pos = 60 * self.scale
-        start_x_pos = 70 * self.scale
+        start_y_pos = 60
+        start_x_pos = 70
         amount_hosts = len(self.config.gateways) + 1
         client_item = ClientCanvasItem(self.main_window, start_x_pos, start_y_pos)
         host_items = []
         connection_items = []
         for i in range(amount_hosts):
-            start_x_pos += 220 * self.scale
-            host_item = HostCanvasItem(self.main_window, start_x_pos, start_y_pos)
+            start_x_pos += 220
+            host_item = HostCanvasItem(self.main_window, start_x_pos, start_y_pos, self.config.all_host_ips[i])
             host_items.append(host_item)
             if i == 0:
                 connection_item = ConnectionCanvasItem(self.main_window, client_item, host_items[i])
@@ -143,17 +143,17 @@ class ItemsGenerator(LoggerMixin):
 
 
 class ClientAdapter:
-    """."""  # need Costas to help me fill out docstring
+    """Adapts a client object to a representational state for the GUI."""
 
     def __init__(self, item):
-        """____________."""
+        """Instantiates the ClientAdapter object."""
         self.item = item
 
     def __str__(self):
         return 'Client'
 
     def stop(self):
-        """____________."""
+        """Stops."""
         self.item.dim()
         return True
 
@@ -162,8 +162,7 @@ class TunnelAdapter:
     """Adapts a tunnel object to a representational state for the GUI."""
 
     def __init__(self, object_, items):
-        """____________."""
-        # self.__class__.__name__ = 'Tunnel'
+        """Instantiates the TunnelAdapter object."""
         self.object_ = object_
         self.items = items
 
@@ -171,7 +170,7 @@ class TunnelAdapter:
         return 'Tunnel'
 
     def stop(self):
-        """____________."""
+        """Stops."""
         for item in self.items:
             item.hide()
         return self.object_.stop()
@@ -181,14 +180,14 @@ class HostAdapter:
     """Adapts a host object to a representational state for the GUI."""
 
     def __init__(self, items):
-        """____________."""
+        """Instantiates the HostAdapter object."""
         self.items = items
 
     def __str__(self):
         return 'Host'
 
     def stop(self):
-        """____________."""
+        """Stops."""
         for host in self.items:
             host.dim()
         return True
@@ -198,14 +197,14 @@ class AgentAdapter:
     """Adapts an agent object to a representational state for the GUI."""
 
     def __init__(self, item):
-        """____________."""
+        """Instantiates the AgentAdapter object."""
         self.item = item
 
     def __str__(self):
         return 'Agent'
 
     def stop(self):
-        """____________."""
+        """Stops."""
         self.item.dim()
         return True
 
@@ -214,13 +213,12 @@ class SetupLink(LoggerMixin):  # pylint: disable=too-many-instance-attributes
     """Establishes a connection to target destination host via intermediaries by starting various objects.
 
     This function also passes the instantiated objects to the StateManager, which
-    will stop the Tunnel and Instructor after a KeyboardInterrupt (by the user
-    or by the program (in FILE mode)).
+    will stop the Tunnel and Instructor after a KeyboardInterrupt.
     """
 
     def __init__(self, state, transfer_agent, tunnel, bootstrap_agent, instructor,  # pylint: disable=too-many-arguments
                  client_item, host_items, agent_item, connection_items):
-        """Initializes the SetupLink object.
+        """Instantiates the SetupLink object.
 
         Args:
             state (StateManager): An instantiated StateManager object.
@@ -236,7 +234,7 @@ class SetupLink(LoggerMixin):  # pylint: disable=too-many-instance-attributes
         """
         super().__init__()
         self.is_terminate_query_scp = False
-        self.terminate_query_ssh = False
+        self.is_terminate_query_ssh = False
         self.state = state
         self.transfer_agent = transfer_agent
         self.tunnel = tunnel
@@ -248,7 +246,7 @@ class SetupLink(LoggerMixin):  # pylint: disable=too-many-instance-attributes
         self.connection_items = connection_items
 
     def start(self):
-        """."""
+        """Starts setting up link."""
         self.start_client()
         self.start_transfer_agent()
         self.start_tunnel()
@@ -257,27 +255,25 @@ class SetupLink(LoggerMixin):  # pylint: disable=too-many-instance-attributes
 
     def _query_connection_transfer_agent(self):
         self._logger.debug('querying for authenticated hosts...')
-        while True:
-            if len(self.transfer_agent.authenticated_hosts) == len(self.transfer_agent.all_hosts):
+        while not self.is_terminate_query_scp:
+            if len(self.transfer_agent.authenticated_hosts) == len(self.transfer_agent.all_host_ips):
                 self.agent_item.move()
                 break
-            if self.is_terminate_query_scp:
-                break
 
     def _query_ssh_proxyjump_connection(self):
         """Shows a connection canvas item every time the tunnel.authenticated_hosts is appended with a new host."""
         self._logger.debug('querying for authenticated hosts...')
         index = 0
-        while index < len(self.tunnel.all_hosts):
-            if index == 0 and self.tunnel.all_hosts[index] in self.tunnel.authenticated_hosts:  # shows the 1st conn.
+        while index < len(self.tunnel.all_host_ips):
+            if index == 0 and self.tunnel.all_host_ips[index] in self.tunnel.authenticated_hosts:  # shows the 1st conn.
                 self.connection_items[index].show()
                 index += 1
-            if index > 0 and self.tunnel.all_hosts[index] in self.tunnel.authenticated_hosts:  # shows the nth conn.
+            elif index > 0 and self.tunnel.all_host_ips[index] in self.tunnel.authenticated_hosts:  # shows the nth conn
                 self.connection_items[index].show()
                 index += 1
-            if self.is_terminate_query_scp:
+            if self.is_terminate_query_ssh:
                 break
-            sleep(0.1)  # otherwise, tk behaves erratic; drawing 2 moving canvas items simultaneously
+            sleep(0.2)  # otherwise, tk might behaves erratic; drawing 2 moving canvas items simultaneously
 
     def start_client(self):
         """Shows the client item.
@@ -289,7 +285,7 @@ class SetupLink(LoggerMixin):  # pylint: disable=too-many-instance-attributes
         self.client_item.setup_ok()
 
     def start_transfer_agent(self):
-        """."""
+        """Starts transferring Agent to destination host."""
         # the TransferAgent object is a disposable one-trick pony
         # no need to invoke state.add_object()
         self.agent_item.show()
@@ -299,14 +295,14 @@ class SetupLink(LoggerMixin):  # pylint: disable=too-many-instance-attributes
             self.agent_item.transfer_nok()
             self.is_terminate_query_scp = True
             raise SetupFailed(self.transfer_agent)
-        while not thread.is_alive():
-            break
-        self.agent_item.transfer_ok()
+        while thread.is_alive():
+            sleep(1)
+        self.agent_item.transfer_ok()  # <<< Tim, this needs attention...
         self._logger.info('Agent has been transferred securely to destination host')
-        sleep(0.15)  # to keep following the movements of items on canvas possible
+        sleep(0.5)  # give the user some time to follow all activity on the canvas
 
     def start_tunnel(self):
-        """."""
+        """Starts setting up the Tunnel with forwarded connections used by Instructor."""
         self.state.add_object(TunnelAdapter(self.tunnel, self.connection_items))
         self.state.add_object(HostAdapter(self.host_items))
         thread = threading.Thread(target=self._query_ssh_proxyjump_connection)
@@ -315,18 +311,19 @@ class SetupLink(LoggerMixin):  # pylint: disable=too-many-instance-attributes
             for conn, host in zip(self.connection_items, self.host_items):
                 conn.setup_nok()
                 host.setup_nok()
-            self.terminate_query_ssh = True
+            self.is_terminate_query_ssh = True
             raise SetupFailed(self.tunnel)
         self._logger.info('Tunnel has been opened...')
-        while not thread.is_alive():
-            break
+        while thread.is_alive():
+            sleep(1)
         for conn, host in zip(self.connection_items, self.host_items):
+            sleep(0.1)
             conn.setup_ok()
             host.setup_ok()
-        sleep(0.15)  # to keep following the movements of items on canvas possible
+        sleep(0.5)  # give the user some time to follow all activity on the canvas
 
     def start_bootstrap_agent(self):
-        """."""
+        """Starts bootstrapping the agent by executing agent module on destination host."""
         # the BootstrapAgent object is a disposable one-trick pony
         if not self.bootstrap_agent.start():
             self.agent_item.setup_nok()
@@ -335,27 +332,25 @@ class SetupLink(LoggerMixin):  # pylint: disable=too-many-instance-attributes
         self.agent_item.setup_ok()
 
     def start_instructor(self):
-        """."""
+        """Starts the Instructor and checks if the Agent can be reached."""
         self.state.add_object(self.instructor)
         self.state.add_object(AgentAdapter(self.agent_item))
         if not self.instructor.start():
-            self.agent_item.setup_nok()
             raise SetupFailed(self.instructor)
         self._logger.info('Instructor has been executed')
-        self.agent_item.setup_ok()
 
 
 class StateVisualiser:  # pylint: disable=too-few-public-methods
     """Shows the state of the encrypted tunnel by a banner and by visualising the packet flow."""
 
     def __init__(self, heartbeat, animated_packet, status_item):
-        """Initializes the StateVisualiser object.
+        """Instantiates the StateVisualiser object.
 
         Args:
-            animated_packet: __________
+            animated_packet: ...
             heartbeat: An instantiated Heartbeat context manager object which determines periodically the
                         state of the tunnel.
-            status_item: __________
+            status_item: ...
 
         """
         self.animated_packet = animated_packet
@@ -363,7 +358,7 @@ class StateVisualiser:  # pylint: disable=too-few-public-methods
         self.status_item = status_item
 
     def start(self):
-        """_____________."""
+        """Start continuously determining the state of the tunnel."""
         self.status_item.show('established')
         self.animated_packet.start()
         while not self.heartbeat.terminate:
