@@ -33,8 +33,9 @@ Main code for various tkinter canvas items.
 
 import uuid
 import re
-from abc import ABC, abstractmethod
+from time import sleep
 from itertools import cycle
+from abc import ABC, abstractmethod
 
 __author__ = '''Vincent Schouten <inquiry@intoreflection.co>'''
 __docformat__ = '''google'''
@@ -65,16 +66,21 @@ NOK_COLOUR = 'red'
 
 
 class CanvasItem(ABC):
-    """Enforces methods to be implemented and provide other methods for the subclasses of this object to operate."""
+    """Enforces methods to be implemented for the subclassed objects."""
 
     def __init__(self, main_window):
+        """Instantiates the CanvasItem object.
+
+        Args:
+            main_window:
+        """
         self.main_window = main_window
         self.canvas_landscape = main_window.main_frame.canvas_frame.canvas_landscape
         self.canvas_status = main_window.main_frame.canvas_frame.canvas_status
         self.scale = main_window.scale
         self.item_name = self._determine_item_name()
-        self.tag = f"{self.item_name}-{self._uuid}"
-        self.label = 0
+        self.item_tag = f"{self.item_name}-{self._uuid}"
+        self.item_label = 0
 
     @property
     def _uuid(self):
@@ -84,16 +90,16 @@ class CanvasItem(ABC):
     def _determine_item_name(self):
         return re.findall(r'[A-Z](?:[a-z]+|[A-Z]*(?=[A-Z]|$))', type(self).__name__)[0]  # split words at CamelCase
 
-    def _add_label(self):
-        coord_x1, coord_y1, coord_x2, _ = self.canvas_landscape.coords(self.tag)
+    def _add_label(self, item_name):
+        coord_x1, coord_y1, coord_x2, _ = self.canvas_landscape.coords(self.item_tag)
         distance = coord_x2 - coord_x1
         pos_x = coord_x1 + (distance / 2)  # to determine the coordinates of the center of the shape (host/client)
         pos_y = coord_y1 - (10 * self.scale)  # put the label a few pixels above the shape
-        self.label = self.canvas_landscape.create_text(pos_x,
-                                                       pos_y,
-                                                       text=self.item_name,
-                                                       font=('', FONT_SIZE, 'normal'),
-                                                       fill=LABEL_FONT_COLOUR)
+        self.item_label = self.canvas_landscape.create_text(pos_x,
+                                                            pos_y,
+                                                            text=item_name,
+                                                            font=('', FONT_SIZE, 'normal'),
+                                                            fill=LABEL_FONT_COLOUR)
 
     @abstractmethod
     def show(self):
@@ -105,21 +111,33 @@ class CanvasItem(ABC):
 
     @abstractmethod
     def setup_ok(self):
-        """________________."""
+        """Colours the canvas item in accordance with an OK state."""
 
     @abstractmethod
     def setup_nok(self):
-        """."""
+        """Colours the canvas item in accordance with an NOK state."""
 
     @abstractmethod
     def dim(self):
-        """."""
+        """Colours the canvas item representing a non-operational state."""
 
 
 class Effect:
-    """."""
+    """Applies effects to the canvas item.
+
+    This class provides methods to colour the canvas item in accordance with the
+    operational state and provide a method to make the canvas item flicker.
+
+    """
 
     def __init__(self, main_window, canvas_item, filling_type):
+        """Instantiates the Effect object.
+
+        Args:
+            main_window:
+            canvas_item:
+            filling_type:
+        """
         self.canvas_landscape = main_window.main_frame.canvas_frame.canvas_landscape
         self.canvas_status = main_window.main_frame.canvas_frame.canvas_status
         self.flicker_index = 0
@@ -178,7 +196,7 @@ class ConnectionCalculator:
     """Calculates all properties that is needed to render a connection between two canvas items."""
 
     def __init__(self, main_window, component_a, component_b):
-        """Initialize the ConnectionCalculator object.
+        """Instantiates the ConnectionCalculator object.
 
         Note: canvas.bbox doesn't return values when the item's state is 'hidden'
 
@@ -189,50 +207,50 @@ class ConnectionCalculator:
 
         """
         self.canvas_landscape = main_window.main_frame.canvas_frame.canvas_landscape
-        self.component_a = component_a.tag
-        self.component_b = component_b.tag
+        self.component_a = self.canvas_landscape.bbox(component_a.item_tag)
+        self.component_b = self.canvas_landscape.bbox(component_b.item_tag)
         self.factor = 0.05
 
     def get_connection_length_inner(self):
         """Returns the distance between two items starting from right side first item to left side second item."""
-        _, _, ax2, _ = self.canvas_landscape.bbox(self.component_a)
-        bx1, _, _, _ = self.canvas_landscape.bbox(self.component_b)
+        _, _, ax2, _ = self.component_a
+        bx1, _, _, _ = self.component_b
         distance = bx1 - ax2
         return distance
 
     def get_connection_length_outer(self):
         """Returns the distance between two items starting from right side first item to left side second item."""
-        ax1, _, _, _ = self.canvas_landscape.bbox(self.component_a)
-        _, _, bx2, _ = self.canvas_landscape.bbox(self.component_b)
+        ax1, _, _, _ = self.component_a
+        _, _, bx2, _ = self.component_b
         distance = bx2 - ax1
         return distance
 
     def get_connection_height(self):
         """Returns the height of a connection based on the height of the item."""
-        _, by1, _, by2 = self.canvas_landscape.bbox(self.component_b)
+        _, by1, _, by2 = self.component_b
         return (by2 - by1) * self.factor  # use bbox to return the bounding box for client & host == more sophisticated
 
     def get_x_pos_connection_right_side(self):
         """Returns the starting position on the X-axis of the connection item."""
-        _, _, ax2, _ = self.canvas_landscape.bbox(self.component_a)
+        _, _, ax2, _ = self.component_a
         return ax2
 
     def get_x_pos_connection_left_side(self):
         """Returns the starting position on the X-axis of the connection item."""
-        ax1, _, _, _ = self.canvas_landscape.bbox(self.component_a)
+        ax1, _, _, _ = self.component_a
         return ax1
 
     def get_y_pos_connection(self):
         """Returns the first position on the Y-axis of the connection item."""
-        _, by1, _, by2 = self.canvas_landscape.bbox(self.component_b)
+        _, by1, _, by2 = self.component_b
         return by1 + ((by2 - by1) / 2)
 
 
 class ClientCanvasItem(CanvasItem):
-    """."""
+    """Creates a canvas item representing a client."""
 
     def __init__(self, main_window, start_pos_x, start_pos_y):
-        """Initialize the ClientCanvasItem object.
+        """Instantiates the ClientCanvasItem object.
 
         Args:
             main_window: _______
@@ -241,7 +259,7 @@ class ClientCanvasItem(CanvasItem):
 
         """
         super().__init__(main_window)
-        self.client_effect = Effect(main_window, self.tag, 'outline')
+        self.client_effect = Effect(main_window, self.item_tag, 'outline')
         self.start_pos_x = start_pos_x
         self.start_pos_y = start_pos_y
         self.width = 90 * self.scale
@@ -257,7 +275,7 @@ class ClientCanvasItem(CanvasItem):
 
     def show(self):  # this code is exactly the same as HostShape
         self.client_effect.flicker()
-        self._add_label()
+        self._add_label(self.item_name)
 
     def hide(self):
         pass
@@ -269,7 +287,7 @@ class ClientCanvasItem(CanvasItem):
                                                            self.start_pos_y + self.height,
                                                            width=THICKNESS_LINE_CLIENT * self.scale,
                                                            outline=NON_OPERATION,
-                                                           tags=self.tag
+                                                           tags=self.item_tag
                                                            # state='hidden'
                                                            )
         self.components.append(component)
@@ -288,7 +306,7 @@ class ClientCanvasItem(CanvasItem):
                                                            start_pos_y + height,
                                                            width=1,
                                                            outline=NON_OPERATION,
-                                                           tags=self.tag
+                                                           tags=self.item_tag
                                                            # state='hidden'
                                                            )
         self.components.append(component)
@@ -306,7 +324,7 @@ class ClientCanvasItem(CanvasItem):
                                                            start_pos_y + height,
                                                            width=THICKNESS_LINE_CLIENT * self.scale,
                                                            outline=NON_OPERATION,
-                                                           tags=self.tag
+                                                           tags=self.item_tag
                                                            # state='hidden'
                                                            )
         self.components.append(component)
@@ -326,7 +344,7 @@ class ClientCanvasItem(CanvasItem):
                                                            fill='',
                                                            width=1,
                                                            outline=NON_OPERATION,
-                                                           tags=self.tag
+                                                           tags=self.item_tag
                                                            # state='hidden'
                                                            )
         self.components.append(component)
@@ -342,23 +360,25 @@ class ClientCanvasItem(CanvasItem):
 
 
 class HostCanvasItem(CanvasItem):
-    """."""
+    """Creates a canvas item representing a host."""
 
-    def __init__(self, main_window, start_pos_x, start_pos_y):
-        """Initialize the HostCanvasItem object.
+    def __init__(self, main_window, start_pos_x, start_pos_y, host_ip):
+        """Instantiates the HostCanvasItem object.
 
         Args:
             main_window: ________
             start_pos_x: ________
             start_pos_y: ________
+            host_ip: ________
 
         """
         super().__init__(main_window)
-        self.host_effect = Effect(main_window, self.tag, 'outline')
+        self.host_effect = Effect(main_window, self.item_tag, 'outline')
         self.start_pos_x = start_pos_x
         self.start_pos_y = start_pos_y
         self.width = 90 * self.scale
         self.height = self.width * 1.4
+        self.host_ip = host_ip
         self._create()
 
     def _create(self):
@@ -372,13 +392,13 @@ class HostCanvasItem(CanvasItem):
                                                host_y2,
                                                width=THICKNESS_LINE_HOST * self.scale,
                                                outline=NON_OPERATION,
-                                               tags=self.tag,
+                                               tags=self.item_tag,
                                                # state='hidden'
                                                )
 
-    def show(self):  # this code is exactly the same as ShapeClient
+    def show(self):
         self.host_effect.flicker()
-        self._add_label()
+        self._add_label(self.host_ip)
 
     def hide(self):
         pass
@@ -394,21 +414,21 @@ class HostCanvasItem(CanvasItem):
 
 
 class AgentCanvasItem(CanvasItem):
-    """."""
+    """Creates a canvas item representing an Agent."""
 
-    def __init__(self, main_window, client, host):
-        """Initialize the AgentCanvasItem object.
+    def __init__(self, main_window, client_canvas_item, host_canvas_item):
+        """Instantiates the AgentCanvasItem object.
 
         Args:
             main_window: _______
-            client: _______
-            host: _______
+            client_canvas_item: _______
+            host_canvas_item: _______
 
         """
         super().__init__(main_window)
-        self.agent_effect = Effect(main_window, self.tag, 'outline')
-        self.coords_client = self.canvas_landscape.coords(client.tag)
-        self.coords_host = self.canvas_landscape.coords(host.tag)
+        self.agent_effect = Effect(main_window, self.item_tag, 'outline')
+        self.coords_client = self.canvas_landscape.coords(client_canvas_item.item_tag)
+        self.coords_host = self.canvas_landscape.coords(host_canvas_item.item_tag)
         self.terminate = False
         self.distance_index = 0
         self.distance = 0
@@ -432,61 +452,66 @@ class AgentCanvasItem(CanvasItem):
                                                    fill=BACKGROUND_CANVAS,
                                                    width=THICKNESS_LINE_AGENT * self.scale,
                                                    outline=NON_OPERATION,
-                                                   tags=self.tag,
+                                                   tags=self.item_tag,
                                                    # state='hidden'
                                                    )
 
-        def _place_agent_on_position_client():
-            ax1, ay1, _, ay2 = self.canvas_landscape.coords(self.tag)
-            _, _, cx2, _ = self.coords_client
-            self.distance = ax1 - cx2  # here comes explanation
-            start_pos_x1 = cx2  # here comes explanation
-            pos_y1 = ay1
-            pos_y2 = ay2
-            self.canvas_landscape.coords(self.tag,
-                                         start_pos_x1,
-                                         pos_y1,
-                                         start_pos_x1 + self.width,
-                                         pos_y2)
+        # def _place_agent_on_position_client():
+        #     ax1, ay1, _, ay2 = self.canvas_landscape.coords(self.item_tag)
+        #     _, _, cx2, _ = self.coords_client
+        #     self.distance = ax1 - cx2  # here comes explanation
+        #     start_pos_x1 = cx2  # here comes explanation
+        #     pos_y1 = ay1
+        #     pos_y2 = ay2
+        #     self.canvas_landscape.coords(self.item_tag,
+        #                                  start_pos_x1,
+        #                                  pos_y1,
+        #                                  start_pos_x1 + self.width,
+        #                                  pos_y2)
 
         _create_agent_derived_from_host()
-        _place_agent_on_position_client()
+        # _place_agent_on_position_client()  # disabled on 2021-05-17
 
     def show(self):
-        self.canvas_landscape.itemconfig(self.tag, state='normal')
+        self.canvas_landscape.itemconfig(self.item_tag, dash=(9, 9), width=1, state='normal')
 
     def hide(self):
         pass
 
     def move(self):
-        """Moves the Agent-item (with its outline in a dash pattern) from Client item to destination Host item."""
+        """Moves the Agent-item from Client item to destination Host item.
 
-        def _animate():
-            if not self.terminate:
-                # print(self.distance_index)
-                self.distance_index -= self.distance / 10
-                self.canvas_landscape.move(self.tag, self.distance / 10, 0)
-                if self.distance_index <= 0:
-                    self._add_label()
-                    self.terminate = True
-                self.canvas_landscape.after(8, _animate)
+        Agent item's outline is in a dash pattern).
+        """
 
-        self.canvas_landscape.itemconfig(self.tag, dash=(5, 5), width=1)
-        self.distance_index = self.distance
-        _animate()  # note, the animation will be threaded, so after invoking this method, it continues immediately
+        # def _animate():
+        #     hx1, _, hx2, _ = self.coords_host
+        #     width_host = hx2-hx1
+        #     total_distance = self.distance - width_host / 2
+        #     stepper = (total_distance - self.width) / 10
+        #     while self.distance_index > 0:
+        #         self.canvas_landscape.move(self.item_tag, stepper, 0)
+        #         self.distance_index -= stepper
+        #         sleep(.1)
+        #
+        #     self._add_label(self.item_name)
+
+        # self.distance_index = self.distance
+        # _animate()  # disabled on 2021-05-17 due to issues
+        self.canvas_landscape.itemconfig(self.item_tag, dash=(5, 5), width=1, state='normal')
 
     def transfer_ok(self):
         """Turns the outline with its dash pattern into a solid line."""
-        self.canvas_landscape.itemconfig(self.tag,
+        self.canvas_landscape.itemconfig(self.item_tag,
                                          dash=(1, 1),  # default to normal, it was dash=(5, 5)
                                          outline=NON_OPERATION,
                                          width=THICKNESS_LINE_AGENT * self.scale)
 
     def transfer_nok(self):
         """Colours the outline, still with a dash pattern, red."""
-        self.canvas_landscape.itemconfig(self.tag, dash=(5, 5))  # default to normal, it was dash=(5, 5)
-        self.canvas_landscape.itemconfig(self.tag, outline=NOK_COLOUR)
-        self.canvas_landscape.itemconfig(self.tag, width=THICKNESS_LINE_AGENT * self.scale)
+        self.canvas_landscape.itemconfig(self.item_tag, dash=(5, 5))
+        self.canvas_landscape.itemconfig(self.item_tag, outline=NOK_COLOUR)
+        self.canvas_landscape.itemconfig(self.item_tag, width=THICKNESS_LINE_AGENT * self.scale)
 
     def setup_ok(self):
         self.agent_effect.setup_ok()
@@ -499,10 +524,10 @@ class AgentCanvasItem(CanvasItem):
 
 
 class ConnectionCanvasItem(CanvasItem):  # pylint: disable=too-many-instance-attributes
-    """."""
+    """Creates a canvas item representing a connection."""
 
     def __init__(self, main_window, host_item_1, host_item_2):
-        """Initialize the ConnectionCanvasItem object.
+        """Instantiates the ConnectionCanvasItem object.
 
         Args:
             main_window: ________
@@ -512,9 +537,9 @@ class ConnectionCanvasItem(CanvasItem):  # pylint: disable=too-many-instance-att
         """
         super().__init__(main_window)
         self.connection_calculator = ConnectionCalculator(main_window, host_item_1, host_item_2)  # called composition
-        self.connection_effect = Effect(main_window, self.tag, 'fill')
-        self.item_id_1 = host_item_1.tag
-        self.item_id_2 = host_item_2.tag
+        self.connection_effect = Effect(main_window, self.item_tag, 'fill')
+        self.item_id_1 = host_item_1.item_tag
+        self.item_id_2 = host_item_2.item_tag
         self.start_pos_x = self.connection_calculator.get_x_pos_connection_right_side()
         self.pos_y = self.connection_calculator.get_y_pos_connection()
         self.distance = self.connection_calculator.get_connection_length_inner()
@@ -530,7 +555,7 @@ class ConnectionCanvasItem(CanvasItem):  # pylint: disable=too-many-instance-att
                                                      self.pos_y - 10,
                                                      fill=NON_OPERATION,
                                                      width=THICKNESS_LINE_TUNNEL * self.scale,
-                                                     tags=self.tag,
+                                                     tags=self.item_tag,
                                                      # state='hidden'
                                                      )
         self.components.append(top_line)
@@ -540,37 +565,39 @@ class ConnectionCanvasItem(CanvasItem):  # pylint: disable=too-many-instance-att
                                                         self.pos_y + 10,
                                                         fill=NON_OPERATION,
                                                         width=THICKNESS_LINE_TUNNEL * self.scale,
-                                                        tags=self.tag,
+                                                        tags=self.item_tag,
                                                         # state='hidden'
                                                         )
         self.components.append(bottom_line)
 
     def show(self):
-        self.canvas_landscape.itemconfig(tagOrId=self.tag, state='normal')
+        self.canvas_landscape.itemconfig(self.item_tag, state='normal')
 
         def animate():
-            if not self.terminate:
+            self.distance_index = self.distance
+            pos_x_stepper = self.start_pos_x
+            stepper = self.distance / 8
+            while self.distance_index >= 0:
                 self.canvas_landscape.coords(self.components[0],
                                              self.start_pos_x,
                                              self.pos_y - 10,
-                                             self.start_pos_x + self.distance_index,
+                                             pos_x_stepper,
                                              self.pos_y - 10)
                 self.canvas_landscape.coords(self.components[1],
                                              self.start_pos_x,
                                              self.pos_y + 10,
-                                             self.start_pos_x + self.distance_index,
+                                             pos_x_stepper,
                                              self.pos_y + 10)
-                self.distance_index += self.distance / 10
-                self.canvas_landscape.after(8, animate)
-                if self.distance_index >= self.distance:
-                    self._add_label()
-                    self.terminate = True
+                self.distance_index -= stepper
+                pos_x_stepper += stepper
+                sleep(.01)
+            self._add_label(self.item_name)
 
-        animate()  # note, the animation will be threaded, so after invoking this method, the def continues immediately
+        animate()
 
     def hide(self):
-        self.canvas_landscape.itemconfig(tagOrId=self.tag, state='hidden')
-        self.canvas_landscape.itemconfig(tagOrId=self.label, state='hidden')
+        self.canvas_landscape.itemconfig(self.item_tag, state='hidden')
+        self.canvas_landscape.itemconfig(self.item_label, state='hidden')
 
     def setup_ok(self):
         self.connection_effect.setup_ok()
@@ -583,12 +610,18 @@ class ConnectionCanvasItem(CanvasItem):  # pylint: disable=too-many-instance-att
 
 
 class StatusBannerCanvasItem(CanvasItem):
-    """."""
+    """Creates a status banner."""
 
     def __init__(self, main_window):
+        """Instantiates the StatusBannerCanvasItem object.
+
+        Args:
+            main_window:
+
+        """
         super().__init__(main_window)
-        self.tag_text_item = f"{self.tag}-1"
-        self.tag_box_item = f"{self.tag}-2"
+        self.tag_text_item = f"{self.item_tag}-1"
+        self.tag_box_item = f"{self.item_tag}-2"
         self._create_text()
         self._create_box()
         self.text_effect = Effect(main_window, self.tag_text_item, 'fill')
@@ -622,9 +655,6 @@ class StatusBannerCanvasItem(CanvasItem):
                                             width=THICKNESS_PACKET,
                                             tags=self.tag_box_item)
 
-    # def create(self):
-    #     self._create_box(self._create_text())
-
     def _show_text(self, state=None):
         if state is None:
             self.canvas_status.itemconfig(self.tag_text_item, state='normal')
@@ -671,23 +701,12 @@ class StatusBannerCanvasItem(CanvasItem):
         self.text_effect.dim()
         self.box_effect.dim()
 
-    # def setup_ok(self, filling_type=None):
-    #     """Colours the canvas item to state OK (green)."""
-    #     arguments = {'fill': OK_COLOUR}
-    #     self.canvas.itemconfig(self.text_item, **arguments)
-    #     arguments = {'outline': OK_COLOUR}
-    #     self.canvas.itemconfig(self.box_item, **arguments)
-    #
-    # def dim(self, filling_type=None):
-    #     self.status.itemconfig(self.tag, fill=NON_OPERATION)
-    #     self.status.itemconfig(self.tag, outline=NON_OPERATION)
-
 
 class PacketCanvasItem(CanvasItem):
     """Creates a visualised TCP packet."""
 
     def __init__(self, main_window, connection_items):
-        """Initialize the PacketShape object.
+        """Instantiates the PacketCanvasItem object.
 
         Args:
             main_window: _________
@@ -711,13 +730,13 @@ class PacketCanvasItem(CanvasItem):
                                                outline=PACKET_OUTLINE_COLOUR,
                                                width=THICKNESS_LINE_PACKET,
                                                # state='hidden',
-                                               tags=self.tag)
+                                               tags=self.item_tag)
 
     def show(self):
-        self.canvas_landscape.itemconfig(self.tag, state='normal')
+        self.canvas_landscape.itemconfig(self.item_tag, state='normal')
 
     def hide(self):
-        self.canvas_landscape.itemconfig(self.tag, state='hidden')
+        self.canvas_landscape.itemconfig(self.item_tag, state='hidden')
 
     def setup_ok(self):
         pass
