@@ -31,16 +31,16 @@ Main code for tkinter windows.
 
 """
 
-from dataclasses import dataclass
 import os
+import inspect
+import threading
+from dataclasses import dataclass
 import tkinter as tk
 from tkinter import DISABLED, NORMAL
 from tkinter.filedialog import askopenfilename
-import inspect
-import threading
+from powermolegui.lib.logging import LoggerMixin
 from powermolegui.lib.application import application
 from powermolegui.lib.frames import MainFrame, CommandFrame
-from powermolegui.lib.logging import LoggerMixin
 from powermolegui.lib.helpers import ItemsGenerator, parse_configuration_file
 
 __author__ = '''Vincent Schouten <inquiry@intoreflection.co>'''
@@ -88,39 +88,39 @@ class MainWindow(tk.Tk, LoggerMixin):  # pylint: disable=too-many-instance-attri
     """
 
     def __init__(self, *args, **kwargs):
-        """Initializes the main window object."""
+        """Instantiates the main window object."""
         tk.Tk.__init__(self, *args, **kwargs)
         LoggerMixin.__init__(self)
         self.scale = 0
+        self._script_path = self._determine_script_path()
         self._set_title_icon()
         self._set_size_window()
         self._query_windowingsystem()
-        self.menu_bars = None  # holds the Tk menu bars
+        self._menu_bars = None  # holds the Tk menu bars
         self._create_menu()
         self._bind_to_event()
+        self._path_config_file = None  # holds the path to the configuration file; set by _config_file_dialog()
+        self.protocol("WM_DELETE_WINDOW", self.close_window)
         self.main_frame = MainFrame(self, self.scale)
         self.main_frame.pack(side="top", fill="both", expand=True)
         self.main_frame.config(highlightthickness=2)
+        self._set_scrollregion(init=True)
+        self.logging_win_handler = None  # holds the (instantiated) logger handler
         self.command_window = None  # holds a TopLevel widget object proving interface for user for sending commands
         self.transfer_window = None  # IN DEVELOPMENT
-        self.set_scrollregion(init=True)
-        self.should_terminate_application = False  # set to True by stop_powermole(); if True, application will clean up
-        self.is_application_busy = False  # set to True when the application runs
-        self.protocol("WM_DELETE_WINDOW", self.close_window)
-        self.path_config_file = None  # holds the path to the configuration file; set by _config_file_dialog()
+        self.should_terminate_application = False  # if True, the application will stop running and clean up
         self.configuration = None  # holds configuration parameters; set by show_config_graphics()
         self.canvas_items = None  # holds canvas items; generated by (Canvas)ItemGenerator
         self.instructor = None  # holds *Instructor object which provides 'send command' and 'send file' functions
 
     def _determine_script_path(self):  # pylint: disable=no-self-use
-        running_script = inspect.getframeinfo(inspect.currentframe()).filename
+        running_script = inspect.getframeinfo(inspect.currentframe()).filename  # /powermolegui/lib/windows.py
         running_script_dir = os.path.dirname(os.path.abspath(running_script))
-        return running_script_dir
+        return os.path.dirname(running_script_dir)  # /powermolegui
 
     def _set_title_icon(self):
         self.title("powermole")
-        parent = os.path.dirname(self._determine_script_path())  # from /powermolegui/lib/ to /powermolegui/
-        path_file = os.path.join(parent, 'icon', 'application_icon_tunnel.png')
+        path_file = os.path.join(self._script_path, 'icon', 'application_icon_tunnel.png')
         # https://stackoverflow.com/questions/11176638/tkinter-tclerror-error-reading-bitmap-file
         img = tk.PhotoImage(file=path_file)
         self.iconphoto(True, img)
@@ -128,16 +128,16 @@ class MainWindow(tk.Tk, LoggerMixin):  # pylint: disable=too-many-instance-attri
     def _set_size_window(self):
         screen_width = self.winfo_screenwidth()  # width of the computer screen
         screen_height = self.winfo_screenheight()  # height of the computer screen
-        self.scale = determine_scale(screen_width)
+        self.scale = determine_scale(screen_width)  # scale is nowhere used because it's in development
         win_width = WINDOW_WIDTH * self.scale  # width of the main window
         win_height = WINDOW_HEIGHT * self.scale  # height of the main window
         start_x = (screen_width / 2) - (win_width / 2)
         start_y = (screen_height / 2) - (win_height / 2)
-        self.geometry('%dx%d+%d+%d' % (win_width, win_height, start_x, start_y))
+        self.geometry(f'{win_width}x{win_height}+{int(start_x)}+{int(start_y)}')
         self.resizable(True, True)
         # self._logger.info("screen size is: %s x %s", (ws, hs))  # can't work as logger is instantiated later
-        print("screen size is: %s x %s" % (screen_width, screen_height))
-        print("window size: %s x %s" % (win_width, win_height))
+        print(f"screen size is: {screen_width} x {screen_height}")
+        print(f"window size: {win_width} x {win_height}")
 
     def _query_windowingsystem(self):
         print(f"windowing system: {self.tk.call('tk', 'windowingsystem')}")
@@ -150,12 +150,12 @@ class MainWindow(tk.Tk, LoggerMixin):  # pylint: disable=too-many-instance-attri
         send_menu = tk.Menu(menubar)
         logging_menu = tk.Menu(menubar)
         quit_menu = tk.Menu(menubar)
-        self.menu_bars = CollectionMenuBars(file_menu, execution_menu, send_menu, logging_menu, quit_menu)
+        self._menu_bars = CollectionMenuBars(file_menu, execution_menu, send_menu, logging_menu, quit_menu)
 
         file_menu.add_command(label='Open', command=self.config_file_dialog)
-        file_menu.entryconfig('Open', accelerator='Ctrl+O')
+        file_menu.entryconfig('Open', accelerator='Ctrl+O', state=NORMAL)
         file_menu.add_command(label='Open Recent', command=self.retrieve_recently_opened)
-        file_menu.entryconfig('Open Recent', accelerator='Ctrl+T')
+        file_menu.entryconfig('Open Recent', accelerator='Ctrl+T', state=NORMAL)
         menubar.add_cascade(label='File', menu=file_menu)
 
         execution_menu.add_command(label='Run Application', command=self.run_application)
@@ -170,18 +170,15 @@ class MainWindow(tk.Tk, LoggerMixin):  # pylint: disable=too-many-instance-attri
         send_menu.entryconfig('Send Command', accelerator='Ctrl+M', state=DISABLED)
         menubar.add_cascade(label='Send', menu=send_menu)
 
-        # @Tim, I don't get one of the menu entries 'checked'. The user can check
-        # *only* Info or Debug. In a later stage, I want to control the log level,
-        # but currently, it's impossible as the LoggingHandler is instantiated before
-        # instantiation of MainWindow()
         var_info = tk.BooleanVar()
-        var_info.set(value=1)
         var_debug = tk.BooleanVar()
         logging_menu.add_checkbutton(label='Info', onvalue=1, offvalue=0, variable=var_info)
         logging_menu.add_checkbutton(label='Debug', onvalue=1, offvalue=0, variable=var_debug)
         menubar.add_cascade(label='Logging', menu=logging_menu)
+        var_info.set(value=1)
         logging_menu.entryconfig('Info', state=DISABLED)
         logging_menu.entryconfig('Debug', state=DISABLED)
+        # logging_win_handler.setLevel(......)  # IN DEVELOPMENT (including the code block above)
 
         quit_menu.add_command(label='Quit', command=self.close_window)
         quit_menu.entryconfig('Quit', accelerator='Ctrl+Q')
@@ -190,13 +187,13 @@ class MainWindow(tk.Tk, LoggerMixin):  # pylint: disable=too-many-instance-attri
         self.config(menu=menubar)  # self == parent = tk.Tk()
 
     def change_state_menu_bar_entry(self, entry, label, state):
-        """Changes the state of the '%' menu bar."""
+        """Changes the state of the menu bar."""
         if entry == 'send':
-            self.menu_bars.send_menu.entryconfig(label, state=state)  # the 'Send File' menu entry is in development
+            self._menu_bars.send_menu.entryconfig(label, state=state)  # the 'Send File' menu entry is in development
         elif entry == 'file':
-            self.menu_bars.file_menu.entryconfig(label, state=state)
+            self._menu_bars.file_menu.entryconfig(label, state=state)
         elif entry == 'execution':
-            self.menu_bars.execution_menu.entryconfig(label, state=state)
+            self._menu_bars.execution_menu.entryconfig(label, state=state)
 
     def _bind_to_event(self):
         self.bind('<Control-o>', lambda e: self.config_file_dialog())
@@ -211,7 +208,7 @@ class MainWindow(tk.Tk, LoggerMixin):  # pylint: disable=too-many-instance-attri
         """Opens interface for the user to send commands to last host and show output."""
         self.command_window = CommandWindow(self)
 
-    def set_scrollregion(self, init=False):
+    def _set_scrollregion(self, init=False):
         """Sets a scroll region that encompasses all the canvas items."""
         self.main_frame.canvas_frame.canvas_landscape.update_idletasks()
         w_height = self.main_frame.canvas_frame.canvas_landscape.winfo_height()
@@ -229,72 +226,50 @@ class MainWindow(tk.Tk, LoggerMixin):  # pylint: disable=too-many-instance-attri
 
     def retrieve_recently_opened(self):  # fix this py_lint error --> inconsistent-return-statements
         """Retrieves the recently opened configuration file stored in /settings."""
-        # if self.is_application_busy:
-        #     return None
-        parent = os.path.dirname(self._determine_script_path())  # from /powermolegui/lib/ to /powermolegui/
-        path_file_recent = os.path.join(parent, 'settings', 'recently_opened_config_file')
+        path_file_recent = os.path.join(self._script_path, 'settings', 'recently_opened_config_file')
         try:
             with open(path_file_recent) as file:
-                self.path_config_file = file.read().rstrip()
+                self._path_config_file = file.read().rstrip()
         except FileNotFoundError:
-            return None
+            pass
         config_thread = threading.Thread(target=self._show_config_graphics)
         config_thread.start()
 
     def _write_to_recently_opened(self, path_config_file):
         """Stores the location to the recently opened configuration file."""
-        parent = os.path.dirname(self._determine_script_path())  # from /powermolegui/lib/ to /powermolegui/
-        path_file_recent = os.path.join(parent, 'settings', 'recently_opened_config_file')
+        path_file_recent = os.path.join(self._script_path, 'settings', 'recently_opened_config_file')
         with open(path_file_recent, 'w') as file:
             file.write(path_config_file)
 
-    def config_file_dialog(self):  # fix this py_lint error --> inconsistent-return-statements
+    def config_file_dialog(self):
         """Shows the file dialog."""
-        # if self.is_application_busy:
-        #     return None
         self.main_frame.canvas_frame.canvas_landscape.delete("all")
         file_types = [('powermole config file', '*.json')]
-        self.path_config_file = askopenfilename(filetypes=file_types)
-        if self.path_config_file:
+        self._path_config_file = askopenfilename(filetypes=file_types)
+        if self._path_config_file:
             config_thread = threading.Thread(target=self._show_config_graphics)
             config_thread.start()
-            self._write_to_recently_opened(self.path_config_file)
+            self._write_to_recently_opened(self._path_config_file)
 
-    def _get_configuration_file(self):
-        """Retrieves the path instance var, parses the config file, and returns config object.
-
-        This method is called by <show_config_graphics>, because in
-        order to create and show the canvas items, it needs
-        information about the real world hosts.
-        """
-        if not self.path_config_file:
-            return None
-        configuration = parse_configuration_file(self.path_config_file)  # import function
-        if not configuration:
-            self.path_config_file = None
-            return None
-        return configuration
-
-    def _show_config_graphics(self):  # fix pylint inconsistent-return-statements
-        """Creates canvas items and shows the landscape based on the config object.
+    def _show_config_graphics(self):
+        """Creates canvas items and shows the landscape based on the config file.
 
         This method is called by _config_file_dialog when
         the user opens ("Open") a powermole configuration file.
         """
-        self.configuration = self._get_configuration_file()
-        if not self.configuration:
-            return None
-        items_generator = ItemsGenerator(self, self.configuration)
-        self.canvas_items = items_generator.create_canvas_items()  # creates all canvas items
-        items_generator.show_landscape(self.canvas_items)
-        self.set_scrollregion()
-        self.change_state_menu_bar_entry('execution', 'Run Application', NORMAL)  # enable menu bar to start/stop appl.
+        if self._path_config_file:  # return True if the variable is set with a path
+            self.configuration = parse_configuration_file(self._path_config_file)  # return configuration object
+            if self.configuration:
+                items_generator = ItemsGenerator(self, self.configuration)
+                self.canvas_items = items_generator.create_canvas_items()  # creates all canvas items
+                items_generator.show_landscape(self.canvas_items)
+                self._set_scrollregion()
+                self.change_state_menu_bar_entry('execution', 'Run Application',
+                                                 NORMAL)  # enable menu bar to start/stop application
 
-    def run_application(self):  # fix this py_lint error --> inconsistent-return-statements
+    def run_application(self):
         """Starts the application."""
-        # if self.is_application_busy:
-        #     return None
-        run_thread = threading.Thread(target=application, args=(self,))
+        run_thread = threading.Thread(target=application, args=(self,), name='running_application')
         run_thread.start()
         self.change_state_menu_bar_entry('file', 'Open', DISABLED)
         self.change_state_menu_bar_entry('file', 'Open Recent', DISABLED)
@@ -314,28 +289,34 @@ class MainWindow(tk.Tk, LoggerMixin):  # pylint: disable=too-many-instance-attri
 
     def close_window(self):
         """Closes the window."""
-        if not self.is_application_busy:
-            self.should_terminate_application = True
-            self.destroy()
+
+        def _is_thread_application_running():
+            result = False
+            for thread in threading.enumerate():
+                if thread.name == 'running_application':
+                    result = True
+            return result
+
+        if _is_thread_application_running():
+            self._logger.info('*** window _cannot_ be closed as Tunnel is operational (press Ctrl+c) ***')
         else:
-            self._logger.info('*** window _cannot_ be closed during setup and operation of the application ***')
-            self._logger.info('*** press control + c to stop application ***')
+            self.destroy()
 
 
 class CommandWindow(tk.Toplevel):
     """Represents an interface for the user to send commands to last host and show output."""
 
     def __init__(self, parent, *args, **kwargs):
-        """Initialize the TopLevel object."""
+        """Instantiates the TopLevel object."""
         super().__init__(*args, **kwargs)
+        self._bind_to_event()
+        self._is_return_pressed = False
+        self._set_size()
         self.instructor = parent.instructor
         self.scale = 0
         self.title("Interface")
-        self._bind_to_event()
-        self._is_return_pressed = False
         self.protocol("WM_DELETE_WINDOW", self.close_window)
         self.sub_command_window = CommandFrame(self)
-        self._set_size()
 
     def _bind_to_event(self):
         self.bind("<Return>", lambda e: self.send_command())
@@ -380,7 +361,7 @@ class TransferWindow(tk.Toplevel):
     """
 
     def __init__(self, *args, **kwargs):
-        """Initialize the TopLevel object."""
+        """Instantiates the TopLevel object."""
         super().__init__(*args, **kwargs)  # with super(), no self as argument is needed
         self.scale = 0
         self.title("Interface")
